![WRF-ICON](https://avatars.githubusercontent.com/u/12666893?s=200&v=4)

The present manual contains basic information about running the Weather Research and Forecasting Model on workstations and clusters of the Department of Meteorology and Geophysics in Vienna.

Besides this basic guide, more detailed information is available too:

[Basic WRF usage, including compilation instructions](WRF/WRF_basic.md).  
[Advanced WRF usage](WRF/WRF_basic.md).  
[Data assimilation with WRF](WRF/WRF_da.md).  
[Workflows for common WRF tasks](WRF/WRF_da.md).  

Table of contents:

[TOC]

## What is WRF

WRF is a community-driven numerical weather prediction model, originally developed in the US in a collaboration between the research community (National Center for Atmospheric Research, [NCAR](https://ncar.ucar.edu), part of the University Corporation for atmospheric Research, [UCAR](https://www.ucar.edu) and the National Weather Service (National Centers for Environmental Prediction, [NCEP](https://www.weather.gov/ncep/) at the National Oceanic and Atmospheric Administration, [NOAA](https://www.noaa.gov/)).

Over the years, WRF evolved into two distinct models. [ARW-WRF](https://www.mmm.ucar.edu/models/wrf) (Advanced Research WRF) is maintained by NCAR and is used by the research community. [WRF-NMM](https://nomads.ncep.noaa.gov/txt_descriptions/WRF_NMM_doc.shtml) is used operationally by the National Weather Service. We use ARW-WRF.

Most of the information about the ARW-WRF is accessible from the [WRF users page](https://www2.mmm.ucar.edu/wrf/users/). The formulation of the model (background theory, numerical aspects, dynamical core, parameterizations) is described in depth in a [Technical description](https://opensky.ucar.edu/islandora/object/opensky:2898), which is periodically updated. The practical use of the model is described in a [User guide](https://www2.mmm.ucar.edu/wrf/users/docs/user_guide_v4/v4.4/contents.html). If you want to acknowledge use of WRF in a manuscript or thesis, and do not like to refer to grey literature, you can use the article by [Skamarock and Klemp (2008)](https://doi.org/10.1016/j.jcp.2007.01.037) as a reference.

NCAR periodically organizes WRF tutorials (one-week workshops for beginners). The [teaching material from the WRF tutorials](https://www2.mmm.ucar.edu/wrf/users/tutorial/tutorial_presentations_2021.htm) is available online and is a great source of information. There is also an [online tutorial](https://www2.mmm.ucar.edu/wrf/OnLineTutorial/index.php) that covers the basics of installing and running WRF.

There is also a [users's forum](https://forum.mmm.ucar.edu/), which can be a source of information on solutions to common problems. However most of the forum posts are about problems, and very few offer useful solutions. Navigating the forum in search of solutions is useless, but landing in the forum from a web source might be useful.

WRF and related programs run as executables on linux machines and clusters. Running WRF requires access to a linux terminal. If you work on Linux or Mac, this is trivial: just open a terminal window. If you work on windows, consider using a linux terminal emulator that supports X11 forwarding (a protocol that enables running interactive graphical applications on a remote server via ssh). There are several alternatives, one option that proved to work well is [MobaXterm](https://mobaxterm.mobatek.net/).

## Getting WRF

### Option 1: Get the WRF executables

If someone else already compiled it on the computer you'll be working with, you just need to:

* copy their WRF run directory.

* replicate exactly their compilation and runtime environment.

### Option 2: Get the source code

The WRF source code is available on [Github](https://github.com/wrf-model/WRF), and there are several ways to get it.

* Recommended: download one of the [official releases](https://github.com/wrf-model/WRF/releases): scroll down to the "Assets" section and choose one of the `v*.tar.gz` or `v*zip` files (not the "Source code" ones; these are incomplete).

    To download while working on the terminal on a remote server, use wget or curl:

    ```sh
    wget "https://github.com/wrf-model/WRF/releases/download/v4.4.2/v4.4.2.tar.gz"
    curl -OL "https://github.com/wrf-model/WRF/archive/refs/tags/v4.4.2.zip"
    ```

    To uncompress the source code, use either of the following (depending on the format):

    ```sh
    tar xzvf v4.4.2.tar.gz
    unzip v4.4.2.zip
    ```

* Clone the repository in a local directory:

    ```sh
    git clone --recurse-submodule https://github.com/wrf-model/WRF.git
    ```

* You can also import WRF from Github into a new empty Gitlab project on [Phaidra](https://gitlab.phaidra.org/explore). To get access to Gitlab, look at the [ZID guidlines](https://zid.univie.ac.at/gitlab/) and send a request via email to `support.phaidra@univie.ac.at`. Once you have access credentials, click on "New Project", then "Import Project", then "Repository by URL". This method might be advisable if you want to use Gitlab for tracking your own changes to the WRF code, but do not want to rely on the official repository.

## Quick start with WRF

### Setting up your environment

You need to make the operating system aware of the software libraries required to compile and run WRF, both at compile time and at run time.

This is done by loading environment modules, with `module load` on srvx1/jet/VSC4, and with `spack load` on VSC5.

It is useful to save the information about a specific environment in a simple bash shell script (for instance: `modules_srvx1.sh`). Then, before compiling or running WRF, type `source modules_srvx1.sh`). See [here](WRF/config_files) for a few examples.

### Compiling

If you already have a compiled version of WRF, go to the next step.

Compiling WRF for an idealized simulation (LES), assuming that you have a properly set software environment:

```sh
./configure
./compile em_les > compile.log 2>&1 &
```

The process is similar for other idealized test cases. Just change the compile targe (e.g., `em_hill2d_x`).

Compiling WRF for a real-case simulation, assuming that you have a properly set software environment:

```sh
./configure
./compile em_real > compile.log 2>&1 &
```

### Running

Running WRF for an idealized simulation (LES), assuming that you have a properly set software environment:

```sh
cd ./test/em_les
./ideal.exe
./wrf.exe
```

For other test cases, compilation might create a `run_me_first.csh` script in the same directory as the executables. If there is one, run it only once, before any other program. It will link any necessary lookup tables needed for the simulation (land-use, parameterizations, etc.).

Running WRF for a real_case simulation, assuming that you have a properly set software environment: 

```sh
cd test/em_real
ln -s $WPS_PATH/met_em* .
./real.exe
./wrf.exe
```

The `met_em*` files linked in this snippet are the outcome of the WRF preprocessing (interpolation of initial and boundary conditions from another model, or from reanalyses, on the WRF grid), and they are supposed to be in a directory, to which the environmental variable `$WPS_PATH` points. 

The WRF preprocessing system (WPS) is a separate set of executables, that can be built only after WRF is successfully compiled. To run WPS for a real-case simulation, getting initial and boundary conditions from ECMWF-IFS data on model levels, you could use a script such as the following. However, the script depends on namelists, variable tables and other settings files being correctly specified. See the detailed info pages for details.

```sh title="Example: wrf-run-script.sh"
#!/bin/bash
set -eu

# Set paths
date=20190726.0000
gribdir=/users/staff/serafin/data/GRIB_IC_for_LAM/ECMWF/TEAMx_convection/

# Run WPS
./geogrid.exe
./link_grib.csh ${gribdir}/${date}/*
./ungrib.exe
./calc_ecmwf_p.exe
./avg_tsfc.exe
mpirun -np 32 ./metgrid.exe

# Archive results and clean up
archive=./archive/TEAMxConv_${date}
mkdir -p ${archive}
mv geo_em.d0?.nc met_em*nc ${archive}
cp namelist.wps geogrid/GEOGRID.TBL.HIRES ${archive}
rm -fr FILE* PRES* TAVGSFC GRIBFILE* metgrid.log.*
```
